==> v1.12.9.zip: wcs/wcs/qommon/misc.py <==
# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import datetime
import re
import os
import time
import httplib
import urllib
import socket
import base64

from quixote import get_publisher, get_session
from quixote.html import htmltext

from qommon import get_cfg, get_logger, ezt
from qommon.errors import ConnectionError

import strftime

from urllib import urlencode, quote
from urllib2 import urlparse
from cStringIO import StringIO


def get_abs_path(s):
    if not s:
        return s
    if s[0] == '/':
        return s
    return os.path.join(get_publisher().app_dir, s)

def get_lasso_server():
    if not get_cfg('sp'):
        return None
    import lasso
    server = lasso.Server(
        get_abs_path(get_cfg('sp')['saml2_metadata']),
        get_abs_path(get_cfg('sp')['privatekey']),
        None, None)

    # Set encryption private key
    encryption_privatekey = get_abs_path(get_cfg('sp').get('encryption_privatekey'))
    if encryption_privatekey and os.path.exists(encryption_privatekey):
        try:
            server.setEncryptionPrivateKey(encryption_privatekey)
        except lasso.Error, error:
            get_logger().warn('Failed to set encryption private key')

    for klp, idp in get_cfg('idp', {}).items():
        try:
            server.addProvider(
                    lasso.PROVIDER_ROLE_IDP,
                    get_abs_path(idp['metadata']),
                    get_abs_path(idp.get('publickey')),
                    get_abs_path(idp.get('cacertchain')))
        except lasso.Error, error:
            if error[0] == lasso.SERVER_ERROR_ADD_PROVIDER_PROTOCOL_MISMATCH:
                continue
            if error[0] == lasso.SERVER_ERROR_ADD_PROVIDER_FAILED:
                continue
            raise

        if hasattr(lasso, 'ENCRYPTION_SYM_KEY_TYPE_DEFAULT'):
            encryption_mode = lasso.ENCRYPTION_MODE_NONE
            if idp.get('encrypt_nameid', False):
                encryption_mode |= lasso.ENCRYPTION_MODE_NAMEID
            provider_t = get_provider(klp)
            provider = server.getProvider(provider_t.providerId)
            if provider is not None:
                provider.setEncryptionMode(encryption_mode)

    return server

def get_provider_label(provider):
    if not provider:
        return None
    if not hasattr(provider, str('getOrganization')):
        return provider.providerId

    organization = provider.getOrganization()
    if not organization:
        return provider.providerId

    name = re.findall("<OrganizationDisplayName.*>(.*?)</OrganizationDisplayName>", organization)
    if not name:
        name = re.findall("<OrganizationName.*>(.*?)</OrganizationName>", organization)
        if not name:
            return provider.providerId
    return htmltext(name[0].decode('utf8').encode(get_publisher().site_charset))

def get_provider(provider_key):
    lp = get_cfg('idp', {}).get(provider_key)
    if not lp:
        raise KeyError()

    import lasso

    publickey_fn = None
    if lp.get('publickey'):
        publickey_fn = get_abs_path(lp['publickey'])
    # cacertchain (not really necessary to get provider label)

    try:
        provider = lasso.Provider(lasso.PROVIDER_ROLE_IDP,
                get_abs_path(lp['metadata']), publickey_fn, None)
    except lasso.Error:
        raise KeyError()

    return provider

def get_provider_key(provider_id):
    return provider_id.replace('://', '-').replace('/', '-').replace('?', '-').replace(':', '-')


xlate = {
    u'\N{ACUTE ACCENT}': "'",
    u'\N{BROKEN BAR}': '|',
    u'\N{CEDILLA}': '{cedilla}',
    u'\N{CENT SIGN}': '{cent}',
    u'\N{COPYRIGHT SIGN}': '{C}',
    u'\N{CURRENCY SIGN}': '{currency}',
    u'\N{DEGREE SIGN}': '{degrees}',
    u'\N{DIAERESIS}': '{umlaut}',
    u'\N{DIVISION SIGN}': '/',
    u'\N{FEMININE ORDINAL INDICATOR}': '{^a}',
    u'\N{INVERTED EXCLAMATION MARK}': '!',
    u'\N{INVERTED QUESTION MARK}': '?',
    u'\N{LATIN CAPITAL LETTER A WITH ACUTE}': 'A',
    u'\N{LATIN CAPITAL LETTER A WITH CIRCUMFLEX}': 'A',
    u'\N{LATIN CAPITAL LETTER A WITH DIAERESIS}': 'A',
    u'\N{LATIN CAPITAL LETTER A WITH GRAVE}': 'A',
    u'\N{LATIN CAPITAL LETTER A WITH RING ABOVE}': 'A',
    u'\N{LATIN CAPITAL LETTER A WITH TILDE}': 'A',
    u'\N{LATIN CAPITAL LETTER AE}': 'Ae',
    u'\N{LATIN CAPITAL LETTER C WITH CEDILLA}': 'C',
    u'\N{LATIN CAPITAL LETTER E WITH ACUTE}': 'E',
    u'\N{LATIN CAPITAL LETTER E WITH CIRCUMFLEX}': 'E',
    u'\N{LATIN CAPITAL LETTER E WITH DIAERESIS}': 'E',
    u'\N{LATIN CAPITAL LETTER E WITH GRAVE}': 'E',
    u'\N{LATIN CAPITAL LETTER ETH}': 'Th',
    u'\N{LATIN CAPITAL LETTER I WITH ACUTE}': 'I',
    u'\N{LATIN CAPITAL LETTER I WITH CIRCUMFLEX}': 'I',
    u'\N{LATIN CAPITAL LETTER I WITH DIAERESIS}': 'I',
    u'\N{LATIN CAPITAL LETTER I WITH GRAVE}': 'I',
    u'\N{LATIN CAPITAL LETTER N WITH TILDE}': 'N',
    u'\N{LATIN CAPITAL LETTER O WITH ACUTE}': 'O',
    u'\N{LATIN CAPITAL LETTER O WITH CIRCUMFLEX}': 'O',
    u'\N{LATIN CAPITAL LETTER O WITH DIAERESIS}': 'O',
    u'\N{LATIN CAPITAL LETTER O WITH GRAVE}': 'O',
    u'\N{LATIN CAPITAL LETTER O WITH STROKE}': 'O',
    u'\N{LATIN CAPITAL LETTER O WITH TILDE}': 'O',
    u'\N{LATIN CAPITAL LETTER THORN}': 'th',
    u'\N{LATIN CAPITAL LETTER U WITH ACUTE}': 'U',
    u'\N{LATIN CAPITAL LETTER U WITH CIRCUMFLEX}': 'U',
    u'\N{LATIN CAPITAL LETTER U WITH DIAERESIS}': 'U',
    u'\N{LATIN CAPITAL LETTER U WITH GRAVE}': 'U',
    u'\N{LATIN CAPITAL LETTER Y WITH ACUTE}': 'Y',
    u'\N{LATIN SMALL LETTER A WITH ACUTE}': 'a',
    u'\N{LATIN SMALL LETTER A WITH CIRCUMFLEX}': 'a',
    u'\N{LATIN SMALL LETTER A WITH DIAERESIS}': 'a',
    u'\N{LATIN SMALL LETTER A WITH GRAVE}': 'a',
    u'\N{LATIN SMALL LETTER A WITH RING ABOVE}': 'a',
    u'\N{LATIN SMALL LETTER A WITH TILDE}': 'a',
    u'\N{LATIN SMALL LETTER AE}': 'ae',
    u'\N{LATIN SMALL LETTER C WITH CEDILLA}': 'c',
    u'\N{LATIN SMALL LETTER E WITH ACUTE}': 'e',
    u'\N{LATIN SMALL LETTER E WITH CIRCUMFLEX}': 'e',
    u'\N{LATIN SMALL LETTER E WITH DIAERESIS}': 'e',
    u'\N{LATIN SMALL LETTER E WITH GRAVE}': 'e',
    u'\N{LATIN SMALL LETTER ETH}': 'th',
    u'\N{LATIN SMALL LETTER I WITH ACUTE}': 'i',
    u'\N{LATIN SMALL LETTER I WITH CIRCUMFLEX}': 'i',
    u'\N{LATIN SMALL LETTER I WITH DIAERESIS}': 'i',
    u'\N{LATIN SMALL LETTER I WITH GRAVE}': 'i',
    u'\N{LATIN SMALL LETTER N WITH TILDE}': 'n',
    u'\N{LATIN SMALL LETTER O WITH ACUTE}': 'o',
    u'\N{LATIN SMALL LETTER O WITH CIRCUMFLEX}': 'o',
    u'\N{LATIN SMALL LETTER O WITH DIAERESIS}': 'o',
    u'\N{LATIN SMALL LETTER O WITH GRAVE}': 'o',
    u'\N{LATIN SMALL LETTER O WITH STROKE}': 'o',
    u'\N{LATIN SMALL LETTER O WITH TILDE}': 'o',
    u'\N{LATIN SMALL LETTER SHARP S}': 'ss',
    u'\N{LATIN SMALL LETTER THORN}': 'th',
    u'\N{LATIN SMALL LETTER U WITH ACUTE}': 'u',
    u'\N{LATIN SMALL LETTER U WITH CIRCUMFLEX}': 'u',
    u'\N{LATIN SMALL LETTER U WITH DIAERESIS}': 'u',
    u'\N{LATIN SMALL LETTER U WITH GRAVE}': 'u',
    u'\N{LATIN SMALL LETTER Y WITH ACUTE}': 'y',
    u'\N{LATIN SMALL LETTER Y WITH DIAERESIS}': 'y',
    u'\N{LEFT-POINTING DOUBLE ANGLE QUOTATION MARK}': '<<',
    u'\N{MACRON}': '_',
    u'\N{MASCULINE ORDINAL INDICATOR}': '{^o}',
    u'\N{MICRO SIGN}': '{micro}',
    u'\N{MIDDLE DOT}': '*',
    u'\N{MULTIPLICATION SIGN}': '*',
    u'\N{NOT SIGN}': '{not}',
    u'\N{PILCROW SIGN}': '{paragraph}',
    u'\N{PLUS-MINUS SIGN}': '{+/-}',
    u'\N{POUND SIGN}': '{pound}',
    u'\N{REGISTERED SIGN}': '{R}',
    u'\N{RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK}': '>>',
    u'\N{SECTION SIGN}': '{section}',
    u'\N{SOFT HYPHEN}': '-',
    u'\N{SUPERSCRIPT ONE}': '{^1}',
    u'\N{SUPERSCRIPT THREE}': '{^3}',
    u'\N{SUPERSCRIPT TWO}': '{^2}',
    u'\N{VULGAR FRACTION ONE HALF}': '{1/2}',
    u'\N{VULGAR FRACTION ONE QUARTER}': '{1/4}',
    u'\N{VULGAR FRACTION THREE QUARTERS}': '{3/4}',
    u'\N{YEN SIGN}': '{yen}'
    }

def latin1_to_ascii (unicrap):
    """This takes a UNICODE string and replaces Latin-1 characters with
        something equivalent in 7-bit ASCII. It returns a plain ASCII string.
        This function makes a best effort to convert Latin-1 characters into
        ASCII equivalents. It does not just strip out the Latin-1 characters.
        All characters in the standard 7-bit ASCII range are preserved.
        In the 8th bit range all the Latin-1 accented letters are converted
        to unaccented equivalents. Most symbol characters are converted to
        something meaningful. Anything not converted is deleted.

        <http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/251871>
    """

    if not isinstance(unicrap, unicode):
        if get_publisher() and get_publisher().site_charset:
            unicrap = unicode(unicrap, get_publisher().site_charset, 'ignore')
        else:
            unicrap = unicode(unicrap, 'iso-8859-1', 'ignore')
    r = ""
    for i in unicrap:
        if xlate.has_key(i):
            r += xlate[i]
        elif ord(i) >= 0x80:
            pass
        else:
            r += str(i)
    return r

simplifyRegex = re.compile(r'''([\s()'"/:\'\.,\|\;\?\&\[\]]+)''')

def simplify(s, space = '-'):
    if s is None: s = ''
    s = latin1_to_ascii(simplifyRegex.sub(' ', s).strip()).replace(' ', space).lower()
    return re.sub(r'\%s+' % space, space, s) # remove consecutive dashes (or whatever)


def get_datetime_language():
    lang = get_cfg('language', {}).get('language', None)
    if lang is None:
        if os.environ.get('LC_TIME'):
            lang = os.environ.get('LC_TIME')[:2]
        elif os.environ.get('LC_ALL'):
            lang = os.environ.get('LC_ALL')[:2]
    return lang

def localstrftime(t):
    if not t:
        return ''
    return strftime.strftime(datetime_format(), t)

def datetime_format():
    format_string = '%Y-%m-%d %H:%M'
    lang = get_datetime_language()
    if lang == 'fr':
        format_string = '%d/%m/%Y %H:%M'
    return format_string

def date_format():
    format_string = '%Y-%m-%d'
    lang = get_datetime_language()
    if lang == 'fr':
        format_string = '%d/%m/%Y'
    return format_string

def get_as_datetime(s):
    for format_string in (datetime_format(), date_format(), '%Y-%m-%d %H:%M', '%Y-%m-%d'):
        try:
            return datetime.datetime.strptime(s, format_string)
        except ValueError:
            pass
    raise ValueError()



def ellipsize(s, length = 30):
    if type(s) is not unicode:
        s = unicode(s, get_publisher().site_charset, 'replace')
    if not s or len(s) < length:
        return s.encode(get_publisher().site_charset)
    return s[:length-5].encode(get_publisher().site_charset) + ' (...)'

def get_month_name(month):
    month_names = [_('January'), _('February'), _('March'), _('April'),
            _('May'), _('June'), _('July'), _('August'),
            _('September'), _('October'), _('November'), _('December') ]
    return month_names[month-1]

def format_time(datetime, formatstring, gmtime = False):
    if not datetime:
        return '?'
    if type(datetime) in (int, float):
        if gmtime:
            datetime = time.gmtime(datetime)
        else:
            datetime = time.localtime(datetime)
    if len(datetime) == 2:
        year, month = datetime
        weekday = None
    elif len(datetime) == 3:
        year, month, day = datetime
        weekday = None
    else:
        year, month, day, hour, minute, second, weekday = datetime[:7]

    weekday_names = [_('Monday'), _('Tuesday'), _('Wednesday'),
            _('Thursday'), _('Friday'), _('Saturday'), _('Sunday')]

    if weekday is not None:
        weekday_name = weekday_names[weekday]
        lower_weekday_name = weekday_name.lower()
        abbr_weekday_name = weekday_name[:3]

    month_name = get_month_name(month)
    lower_month_name = month_name.lower()
    abbr_month_name = month_name[:3]

    return formatstring % locals()

def _http_request(url, method='GET', body=None, headers={}, timeout=None):
    get_publisher().reload_cfg()

    if url.startswith('http://'):
        hostname, query = urllib.splithost(url[5:])
    else:
        hostname, query = urllib.splithost(url[6:])

    if url.startswith('http://'):
        if timeout is None:
            conn = httplib.HTTPConnection(hostname)
        else:
            conn = httplib.HTTPConnection(hostname, timeout=timeout)
    else:
        if timeout is None:
            conn = httplib.HTTPSConnection(hostname)
        else:
            conn = httplib.HTTPSConnection(hostname, timeout=timeout)
    query = query.replace('&amp;', '&')

    try:
        conn.request(method, query, body, headers)
    except socket.gaierror, (err, msg):
        conn.close()
        raise ConnectionError('error while connecting to %s : %s' % (hostname, msg))
    except socket.timeout, err:
        conn.close()
        raise ConnectionError('connection timed out while fetching the page')
    except socket.error, err:
        conn.close()
        raise ConnectionError('error while fetching the page : %s' % err)
    else:
        response = conn.getresponse()
        data = response.read()
        status = response.status
        auth_header = response.getheader('WWW-Authenticate')
        conn.close()

    return response, status, data, auth_header

def http_get_page(url, headers={}, timeout=None):
    return _http_request(url, headers=headers, timeout=timeout)

def http_post_request(url, body=None, headers={}, timeout=None):
    return _http_request(url, 'POST', body, headers, timeout=timeout)

def is_user_admin():
    session = get_session()
    if not session:
        return False
    user = session.get_user_object()
    if not user:
        return False
    if callable(user.is_admin):
        if user.is_admin():
            return True
    elif user.is_admin:
        return True
    return False

def get_variadic_url(url, variables, encode_query=True):
    # substitution in an URL : try to be safe

    def ezt_substitute(template, variables):
        tmpl = ezt.Template()
        tmpl.parse(template)
        fd = StringIO()
        tmpl.generate(fd, variables)
        return fd.getvalue()

    p = urlparse.urlsplit(url)
    scheme, netloc, path, query, fragment = \
            p.scheme, p.netloc, p.path, p.query, p.fragment
    if netloc and '[' in netloc:
        netloc = ezt_substitute(netloc, variables)
    if path and '[' in path:
        if scheme == '' and netloc == '':
            # this happened because the variable was set in the scheme
            # (ex: http[https]://www.example.net) or because the value starts
            # with a variable name (ex: [url]); in that situation we do not
            # quote at all.
            if path.count('/') >= 2:
                # there were no / in the original path (the two / comes from
                # the scheme/netloc separation, this means there is no path
                before_path = ezt_substitute(path, variables)
                p2 = urlparse.urlsplit(before_path)
                scheme, netloc, path = p2.scheme, p2.netloc, p2.path
            else:
                # there is a path, we need to get back to the original URL and
                # split it on the last /, to isolate the path part.
                if '/' in path:
                    before_path, path = path.rsplit('/', 1)
                else:
                    before_path, path = path, ''
                before_path = ezt_substitute(before_path, variables)
                p2 = urlparse.urlsplit(before_path)
                scheme, netloc = p2.scheme, p2.netloc
                if p2.path and not path:
                    path, query = p2.path, p2.query
        if path:
            path = quote(ezt_substitute(path, variables))
    if fragment and '[' in fragment:
        fragment = quote(ezt_substitute(fragment, variables))
    if query and '[' in query:
        p_qs = urlparse.parse_qsl(query)
        if len(p_qs) == 0:
            # this happened because the query string has no key/values,
            # probably because it's a single substitution variable (ex:
            # http://www.example.net/foobar?[query])
            query = ezt_substitute(query, variables)
        else:
            query = []
            for k, v in p_qs:
                if '[' in k:
                    k = ezt_substitute(k, variables)
                if '[' in v:
                    v = ezt_substitute(v, variables)
                query.append((k, v))
            if encode_query:
                query = urlencode(query)
            else:
                query = '&'.join('%s=%s' % (k,v) for (k,v) in query)
    return urlparse.urlunsplit((scheme, netloc, path, query, fragment))


def get_foreground_colour(background_colour):
    """Calculates the luminance of the given colour (six hexadecimal digits)
       and returns an appropriate foreground colour."""
    # luminance coefficients taken from section C-9 from
    # http://www.faqs.org/faqs/graphics/colorspace-faq/
    brightess = int(background_colour[0:2], 16) * 0.212671 + \
            int(background_colour[2:4], 16) * 0.715160 + \
            int(background_colour[4:6], 16) * 0.072169
    if brightess > 128:
        fg_colour = 'black'
    else:
        fg_colour = 'white'
    return fg_colour


def zap_context(msg):
    '''Removes translation context from message'''
    return msg.split('|', 1)[1]


def indent_xml(elem, level=0):
    # in-place prettyprint formatter
    # http://effbot.org/zone/element-lib.htm#prettyprint
    i = "\n" + level*"  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        for elem in elem:
            indent_xml(elem, level+1)
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i
    return elem

==> v1.12.14.zip: wcs/wcs/qommon/misc.py <==
# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import datetime
import re
import os
import time
import httplib
import urllib
import socket
import base64
import json
import unicodedata

from quixote import get_publisher, get_session
from quixote.html import htmltext

from qommon import get_cfg, get_logger, ezt
from qommon.errors import ConnectionError
import qommon.strftime

import strftime

from urllib import urlencode, quote
from urllib2 import urlparse
from cStringIO import StringIO


def get_abs_path(s):
    if not s:
        return s
    if s[0] == '/':
        return s
    return os.path.join(get_publisher().app_dir, s)

def get_lasso_server():
    if not get_cfg('sp'):
        return None
    import lasso
    server = lasso.Server(
        get_abs_path(get_cfg('sp')['saml2_metadata']),
        get_abs_path(get_cfg('sp')['privatekey']),
        None, None)

    # Set encryption private key
    encryption_privatekey = get_abs_path(get_cfg('sp').get('encryption_privatekey'))
    if encryption_privatekey and os.path.exists(encryption_privatekey):
        try:
            server.setEncryptionPrivateKey(encryption_privatekey)
        except lasso.Error, error:
            get_logger().warn('Failed to set encryption private key')

    for klp, idp in get_cfg('idp', {}).items():
        try:
            server.addProvider(
                    lasso.PROVIDER_ROLE_IDP,
                    get_abs_path(idp['metadata']),
                    get_abs_path(idp.get('publickey')),
                    get_abs_path(idp.get('cacertchain')))
        except lasso.Error, error:
            if error[0] == lasso.SERVER_ERROR_ADD_PROVIDER_PROTOCOL_MISMATCH:
                continue
            if error[0] == lasso.SERVER_ERROR_ADD_PROVIDER_FAILED:
                continue
            raise

        if hasattr(lasso, 'ENCRYPTION_SYM_KEY_TYPE_DEFAULT'):
            encryption_mode = lasso.ENCRYPTION_MODE_NONE
            if idp.get('encrypt_nameid', False):
                encryption_mode |= lasso.ENCRYPTION_MODE_NAMEID
            provider_t = get_provider(klp)
            provider = server.getProvider(provider_t.providerId)
            if provider is not None:
                provider.setEncryptionMode(encryption_mode)

    return server

def get_provider_label(provider):
    if not provider:
        return None
    if not hasattr(provider, str('getOrganization')):
        return provider.providerId

    organization = provider.getOrganization()
    if not organization:
        return provider.providerId

    name = re.findall("<OrganizationDisplayName.*>(.*?)</OrganizationDisplayName>", organization)
    if not name:
        name = re.findall("<OrganizationName.*>(.*?)</OrganizationName>", organization)
        if not name:
            return provider.providerId
    return htmltext(name[0].decode('utf8').encode(get_publisher().site_charset))

def get_provider(provider_key):
    lp = get_cfg('idp', {}).get(provider_key)
    if not lp:
        raise KeyError()

    import lasso

    publickey_fn = None
    if lp.get('publickey'):
        publickey_fn = get_abs_path(lp['publickey'])
    # cacertchain (not really necessary to get provider label)

    try:
        provider = lasso.Provider(lasso.PROVIDER_ROLE_IDP,
                get_abs_path(lp['metadata']), publickey_fn, None)
    except lasso.Error:
        raise KeyError()

    return provider

def get_provider_key(provider_id):
    return provider_id.replace('://', '-').replace('/', '-').replace('?', '-').replace(':', '-')

def simplify(s, space='-'):
    if s is None:
        return ''
    if not isinstance(s, unicode):
        if get_publisher() and get_publisher().site_charset:
            s = unicode(s, get_publisher().site_charset, 'ignore')
        else:
            s = unicode(s, 'iso-8859-1', 'ignore')
    s = unicodedata.normalize('NFKD', s).encode('ascii', 'ignore')
    s = re.sub('[^\w\s%s]' % space, '', s).strip().lower()
    s = re.sub('[\s%s]+' % space, space, s)
    return s

def get_datetime_language():
    lang = get_cfg('language', {}).get('language', None)
    if lang is None:
        if os.environ.get('LC_TIME'):
            lang = os.environ.get('LC_TIME')[:2]
        elif os.environ.get('LC_ALL'):
            lang = os.environ.get('LC_ALL')[:2]
    return lang

def localstrftime(t):
    if not t:
        return ''
    return strftime.strftime(datetime_format(), t)

def datetime_format():
    format_string = '%Y-%m-%d %H:%M'
    lang = get_datetime_language()
    if lang == 'fr':
        format_string = '%d/%m/%Y %H:%M'
    return format_string

def date_format():
    format_string = '%Y-%m-%d'
    lang = get_datetime_language()
    if lang == 'fr':
        format_string = '%d/%m/%Y'
    return format_string

def get_as_datetime(s):
    for format_string in (datetime_format(), date_format(), '%Y-%m-%d %H:%M', '%Y-%m-%d'):
        try:
            return datetime.datetime.strptime(s, format_string)
        except ValueError:
            pass
    raise ValueError()



def ellipsize(s, length = 30):
    if type(s) is not unicode:
        s = unicode(s, get_publisher().site_charset, 'replace')
    if not s or len(s) < length:
        return s.encode(get_publisher().site_charset)
    return s[:length-5].encode(get_publisher().site_charset) + ' (...)'

def get_month_name(month):
    month_names = [_('January'), _('February'), _('March'), _('April'),
            _('May'), _('June'), _('July'), _('August'),
            _('September'), _('October'), _('November'), _('December') ]
    return month_names[month-1]

def format_time(datetime, formatstring, gmtime = False):
    if not datetime:
        return '?'
    if type(datetime) in (int, float):
        if gmtime:
            datetime = time.gmtime(datetime)
        else:
            datetime = time.localtime(datetime)
    if len(datetime) == 2:
        year, month = datetime
        weekday = None
    elif len(datetime) == 3:
        year, month, day = datetime
        weekday = None
    else:
        year, month, day, hour, minute, second, weekday = datetime[:7]

    weekday_names = [_('Monday'), _('Tuesday'), _('Wednesday'),
            _('Thursday'), _('Friday'), _('Saturday'), _('Sunday')]

    if weekday is not None:
        weekday_name = weekday_names[weekday]
        lower_weekday_name = weekday_name.lower()
        abbr_weekday_name = weekday_name[:3]

    month_name = get_month_name(month)
    lower_month_name = month_name.lower()
    abbr_month_name = month_name[:3]

    return formatstring % locals()

def _http_request(url, method='GET', body=None, headers={}, timeout=None):
    get_publisher().reload_cfg()

    if url.startswith('http://'):
        hostname, query = urllib.splithost(url[5:])
    else:
        hostname, query = urllib.splithost(url[6:])

    if url.startswith('http://'):
        if timeout is None:
            conn = httplib.HTTPConnection(hostname)
        else:
            conn = httplib.HTTPConnection(hostname, timeout=timeout)
    else:
        if timeout is None:
            conn = httplib.HTTPSConnection(hostname)
        else:
            conn = httplib.HTTPSConnection(hostname, timeout=timeout)
    query = query.replace('&amp;', '&')

    try:
        conn.request(method, query, body, headers)
    except socket.gaierror, (err, msg):
        conn.close()
        raise ConnectionError('error while connecting to %s : %s' % (hostname, msg))
    except socket.timeout, err:
        conn.close()
        raise ConnectionError('connection timed out while fetching the page')
    except socket.error, err:
        conn.close()
        raise ConnectionError('error while fetching the page : %s' % err)
    else:
        response = conn.getresponse()
        data = response.read()
        status = response.status
        auth_header = response.getheader('WWW-Authenticate')
        conn.close()

    return response, status, data, auth_header

def http_get_page(url, headers={}, timeout=None):
    return _http_request(url, headers=headers, timeout=timeout)

def http_post_request(url, body=None, headers={}, timeout=None):
    return _http_request(url, 'POST', body, headers, timeout=timeout)

def is_user_admin():
    session = get_session()
    if not session:
        return False
    user = session.get_user_object()
    if not user:
        return False
    if callable(user.is_admin):
        if user.is_admin():
            return True
    elif user.is_admin:
        return True
    return False

def get_variadic_url(url, variables, encode_query=True):
    # substitution in an URL : try to be safe

    def ezt_substitute(template, variables):
        tmpl = ezt.Template()
        tmpl.parse(template)
        fd = StringIO()
        tmpl.generate(fd, variables)
        return fd.getvalue()

    p = urlparse.urlsplit(url)
    scheme, netloc, path, query, fragment = \
            p.scheme, p.netloc, p.path, p.query, p.fragment
    if netloc and '[' in netloc:
        netloc = ezt_substitute(netloc, variables)
    if path and '[' in path:
        if scheme == '' and netloc == '':
            # this happened because the variable was set in the scheme
            # (ex: http[https]://www.example.net) or because the value starts
            # with a variable name (ex: [url]); in that situation we do not
            # quote at all.
            if path.count('/') >= 2:
                # there were no / in the original path (the two / comes from
                # the scheme/netloc separation, this means there is no path
                before_path = ezt_substitute(path, variables)
                p2 = urlparse.urlsplit(before_path)
                scheme, netloc, path = p2.scheme, p2.netloc, p2.path
            else:
                # there is a path, we need to get back to the original URL and
                # split it on the last /, to isolate the path part.
                if '/' in path:
                    before_path, path = path.rsplit('/', 1)
                else:
                    before_path, path = path, ''
                before_path = ezt_substitute(before_path, variables)
                p2 = urlparse.urlsplit(before_path)
                scheme, netloc = p2.scheme, p2.netloc
                if p2.path and not path:
                    path, query = p2.path, p2.query
        if path:
            path = quote(ezt_substitute(path, variables))
    if fragment and '[' in fragment:
        fragment = quote(ezt_substitute(fragment, variables))
    if query and '[' in query:
        p_qs = urlparse.parse_qsl(query)
        if len(p_qs) == 0:
            # this happened because the query string has no key/values,
            # probably because it's a single substitution variable (ex:
            # http://www.example.net/foobar?[query])
            query = ezt_substitute(query, variables)
        else:
            query = []
            for k, v in p_qs:
                if '[' in k:
                    k = ezt_substitute(k, variables)
                if '[' in v:
                    v = ezt_substitute(v, variables)
                query.append((k, v))
            if encode_query:
                query = urlencode(query)
            else:
                query = '&'.join('%s=%s' % (k,v) for (k,v) in query)
    return urlparse.urlunsplit((scheme, netloc, path, query, fragment))


def get_foreground_colour(background_colour):
    """Calculates the luminance of the given colour (six hexadecimal digits)
       and returns an appropriate foreground colour."""
    # luminance coefficients taken from section C-9 from
    # http://www.faqs.org/faqs/graphics/colorspace-faq/
    brightess = int(background_colour[0:2], 16) * 0.212671 + \
            int(background_colour[2:4], 16) * 0.715160 + \
            int(background_colour[4:6], 16) * 0.072169
    if brightess > 128:
        fg_colour = 'black'
    else:
        fg_colour = 'white'
    return fg_colour


def zap_context(msg):
    '''Removes translation context from message'''
    return msg.split('|', 1)[1]


def indent_xml(elem, level=0):
    # in-place prettyprint formatter
    # http://effbot.org/zone/element-lib.htm#prettyprint
    i = "\n" + level*"  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        for elem in elem:
            indent_xml(elem, level+1)
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i
    return elem


class JSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, time.struct_time):
            return datetime.datetime.utcfromtimestamp(time.mktime(obj)).isoformat() + 'Z'
        # Let the base class default method raise the TypeError
        return json.JSONEncoder.default(self, obj)
